local gsub, strsplit, find, print, UIParent, ipairs, CreateFrame, type, insert = gsub, strsplit, string.find, print, UIParent, ipairs, CreateFrame, type, table.insert;

local TranslationTable = {
    ["Edit"] = {
        ["deDE"] = "Bearbeiten",
    },
    ["Cancel"] = {
        ["deDE"] = "Abbrechen",
    },
    ["Accept"] = {
        ["deDE"] = "Akzeptieren",
    },
}
local Translation = function(Text)
    if type(Text) == "string" then 
        if TranslationTable[Text] then 
            local Lang = GetLocale()
            if TranslationTable[Text][Lang] then 
                return TranslationTable[Text][Lang]
            else 
                return Text
            end 
        else 
            return Text.."NT"
        end 
    end 
end 

local function Release(self)
    self:Hide()
    self:ClearAllPoints()
    self.Released = true
    self:SetAutoFocus(false)
    self:SetText("")
    self:SetMultiLine(false)
    self:SetNumeric(false)
    self:SetScript("OnEnter", nil)
    self:SetScript("OnLeave", nil)
    self:SetScript("OnEscapePressed", nil)
    self:SetScript("OnEnterPressed", nil)
    self:SetScript("OnEditFocusGained", nil)
    self:SetScript("OnEditFocusLost", nil)
    self:ClearFocus()
end

local function EditState(self, Start, Keep)
    if Start then
        self.EditBox:SetFocus()
    else
        if Keep then
            self:SetValue(self.EditBox:GetText(), true)
        end
        self.EditBox:ClearFocus()
        if self.EditBox.UpdateScript then
            self.EditBox.UpdateScript(self.EditBox:GetText())
        end 
    end
end

local function SetValue(self, Value, UserInput)
    if Value then
        if Value ~= self.Value then
            if self.NumericInputFilter then
                Value = gsub(Value, ",", ".")
                local FullNum, DecNum = strsplit(".", Value, 2)
                if FullNum or DecNum then
                    FullNum = gsub((FullNum or ""), "%D", "")
                    if FullNum == "" then
                        FullNum = "0"
                    end
                    DecNum = gsub((DecNum or ""), "%D", "")
                    if find(Value, "-") then
                        Value = "-"
                    else
                        Value = ""
                    end
                    if DecNum == "" or DecNum == "0" then
                        Value = Value .. FullNum
                    else
                        Value = Value .. FullNum .. "." .. DecNum
                    end
                else
                    --print("Questic: Textbox input not a number.")
                    self.EditBox:SetText(self.Value)
                    return nil
                end
            end
            self.Value = Value
            self.EditBox:SetText(self.Value)
            if self.OnValueChanged and UserInput then
				if self.NumericInputFilter then
					self.OnValueChanged(self, tonumber(self.Value))
				else 
					self.OnValueChanged(self, self.Value)				
				end
            end
        end
    else
        self.EditBox:SetText(self.Value)
    end
end

function _Avyr_Global_ADDON_Elements_Lib.WidgetsMixin:CreateBasicTextBox(Parent, MultiLine, HAlign, VAlign, UpdateScript)
    local TextBox = self:CreateFrame(Parent or UIParent)
    TextBox:SetThemedBackdrop()

    TextBox.EditBox = nil
    for i, v in ipairs(_Avyr_Global_ADDON_Elements_Lib.TextBoxes) do
        if v.Released then
            TextBox.EditBox = v
            TextBox.EditBox:SetParent(TextBox)
            TextBox.EditBox:Show()
        end
    end
    if not TextBox.EditBox then
        TextBox.EditBox = CreateFrame("EditBox", nil, TextBox)
        self:SetThemedColor(TextBox.EditBox,"Normal")
        if GetLocale() == "ruRU" then 
            TextBox.EditBox:SetFont("Fonts\\FRIZQT___CYR.TTF", 11)
        else 
            TextBox.EditBox:SetFont("Fonts\\FRIZQT__.TTF", 11)
        end 
        TextBox.EditBox:SetAutoFocus(false)

        insert(_Avyr_Global_ADDON_Elements_Lib.TextBoxes,TextBox.EditBox)
    end
    TextBox.EditBox.Release = Release
    TextBox.EditBox.Released = false
    TextBox.EditBox:SetMultiLine(MultiLine)
    TextBox.EditBox:SetJustifyH(HAlign or "CENTER")
    TextBox.EditBox:SetJustifyV(VAlign or "CENTER")

    --TextBox.EditBox:SetNumeric(Numeric) --Doesnt take , or . or - (only int values)

    TextBox.EditBox:SetScript("OnEnter",
        function(self)
            if not TextBox.Disable and (not TextBox.ActiveCheck or (TextBox.ActiveCheck and TextBox.ActiveCheck())) then
                TextBox.State = "MouseOver"
                TextBox:PresetBorders("MouseOver", self:HasFocus())
            end
        end)

    TextBox.EditBox:SetScript("OnLeave",
        function(self)
            TextBox.State = "Normal"
            TextBox:PresetBorders("Normal", self:HasFocus())
        end)

    TextBox.EditBox:SetScript("OnEscapePressed",
        function(self)
            TextBox:EditState()
        end)

    TextBox.EditBox:SetScript("OnEnterPressed",
        function(self)
            TextBox:EditState(false, true)
        end)

    TextBox.EditBox:SetScript("OnEditFocusGained",
        function(self)
            if not TextBox.Disable and (not TextBox.ActiveCheck or (TextBox.ActiveCheck and TextBox.ActiveCheck())) then
                self:HighlightText()
                TextBox:PresetBorders(TextBox.State, true)
                if TextBox.OnEditStart then
                    TextBox.OnEditStart()
                end
            else
                self:ClearFocus()
            end
        end)

    TextBox.EditBox:SetScript("OnEditFocusLost",
        function(self)
            TextBox:PresetBorders(TextBox.State, false)
            TextBox.EditBox:SetText(TextBox.Value)
            if TextBox.OnEditEnd then
                TextBox.OnEditEnd()
            end
        end)

        if UpdateScript then 
            TextBox.EditBox.UpdateScript = UpdateScript
            TextBox.EditBox:SetScript("OnTextChanged",
            function(self,UserInput)
                if UserInput then 
                    local LTC = 0
                    self:SetScript("OnUpdate",function(self,elapsed)
                        LTC = LTC + elapsed
                        if LTC > 0.6 then 
                            UpdateScript(self:GetText())
                            self:SetScript("OnUpdate",nil)
                        end 
                    end)
                end 
            end)
        else 
            TextBox.EditBox:SetScript("OnUpdate",nil)
            TextBox.EditBox:SetScript("OnTextChanged",nil)
        end 

    TextBox.EditBox:SetPoint("BOTTOMLEFT", self.BorderSize + self.Padding, self.BorderSize + self.Padding)
    TextBox.EditBox:SetPoint("TOPRIGHT", -(self.BorderSize + self.Padding), -(self.BorderSize + self.Padding))

    TextBox.SetValue = SetValue
    TextBox.EditState = EditState
    TextBox.Clear = function(self)
        self.Value = ""
        self:SetValue()
    end
    TextBox.HasFocus = function()
        return TextBox.EditBox:HasFocus()
    end
    TextBox.Value = ""

    return TextBox
end

function _Avyr_Global_ADDON_Elements_Lib.WidgetsMixin:CreateTextBox(Parent, MultiLine)
    local TextBox = self:CreateFrame(Parent)

    TextBox.AcceptBtn = self:CreateButton(TextBox)
    TextBox.AcceptBtn:SetPoint("BOTTOMLEFT")
    TextBox.AcceptBtn:SetPoint("BOTTOMRIGHT", TextBox, "BOTTOM", -self.Padding / 2, 0)
    TextBox.AcceptBtn:SetText(Translation("Accept"),nil,"Highlight")
    TextBox.AcceptBtn:Size(nil, 20)
    TextBox.AcceptBtn.OnClick = function(self)
        if not TextBox.Disable then
            TextBox.EditBox:EditState(false, true)
        end
    end
    TextBox.AcceptBtn:Hide()
    TextBox.AcceptBtn.ActiveCheck = function()
        return not TextBox.Disable
    end

    TextBox.EditCancelBtn = self:CreateButton(TextBox)
    TextBox.EditCancelBtn:SetPoint("BOTTOMRIGHT")
    TextBox.EditCancelBtn:SetPoint("BOTTOMLEFT", TextBox, "BOTTOM", self.Padding / 2, 0)
    TextBox.EditCancelBtn:Size(nil, 20)
    TextBox.EditCancelBtn:SetText(Translation("Edit"),nil,"Highlight")
    TextBox.EditCancelBtn.OnClick = function(self)
        if not TextBox.Disable then
            TextBox.EditBox:EditState(not TextBox.EditBox:HasFocus())
        end
    end
    TextBox.EditCancelBtn.ActiveCheck = function()
        return not TextBox.Disable
    end

    TextBox.EditBox = self:CreateBasicTextBox(TextBox, type(MultiLine) == "number", "LEFT")
    TextBox.EditBox:SetPoint("BOTTOMLEFT", TextBox.AcceptBtn, "TOPLEFT", 0, self.Padding)
    TextBox.EditBox:SetPoint("BOTTOMRIGHT", TextBox.EditCancelBtn, "TOPRIGHT", 0, self.Padding)
    TextBox.EditBox:Size(nil, 20 + (type(MultiLine) == "number" and (MultiLine - 1) * 11 or 0))
    TextBox.EditBox.OnValueChanged = function(_, Value)
        if not TextBox.Disable then
            TextBox:SetValue(Value, true)
        end
    end
    TextBox.EditBox.ActiveCheck = function()
        return not TextBox.Disable
    end

    TextBox.SetValue = function(self, Value, UserInput)
        Value = Value or self.Value
        self.EditBox:SetValue(Value)
        if self.Value ~= Value and UserInput and self.OnValueChanged then
            self.OnValueChanged(self, Value)
        end
    end

    TextBox.EditBox.OnEditStart = function()
        TextBox.AcceptBtn:Show()
        TextBox.EditCancelBtn:SetText(Translation("Cancel"))
    end

    TextBox.EditBox.OnEditEnd = function()
        TextBox.AcceptBtn:Hide()
        TextBox.EditCancelBtn:SetText(Translation("Edit"))
    end

    return TextBox
end
